# ts_all.rb
#
#  Created by James Edward Gray II on 2005-04-26.
#  Copyright 2005 Gray Productions. All rights reserved.
#
#  This is Free Software.  See LICENSE and COPYING for details.

require "test/unit"

require "tc_highline"
require "tc_import"
require "tc_menu"
require "tc_style"
require "tc_color_scheme"
require "tc_string_highline"
require "tc_string_extension"